# bolero

bolero is a nextflow pipeline dedicated to analyse Nanopore sequencing coupled to 5'RACE amplification of HBV RNAs.

## Getting the last updates

To get the last commits from this repository into your fork use the following commands:

```sh
git clone git@gitbio.ens-lyon.fr:xgrand/bolero.git
```

## Getting Started

The pipeline `src/bolero.nf` works a nextflow configuration file `src/nextflow.config`.
The typical command for running the pipeline is as follows:
`nextflow ./src/bolero.nf -c ./src/nextflow.config -profile singularity`

The arguments of this pipeline are described in the table below:

|         Arguments           |                             Description                             | 
|:---------------------------:|:-------------------------------------------------------------------:| 
| -c | configuration file. This parameter should always be `src/nextflow.config`        | 
| -profile      | The profile to use. This can be **docker** or **singularity** to run the pipeline in docker or singularity container respectively. This can also be **psmn** to launch the analysis on the PSMN | 
| --input [path] | Path to the folder containing fast5 files. If skip basecalling option enabled, path to fastq files folder. |
| --adapt [str] | Sequence of 5'RACE adapter. |
| --genome [file] | Path to the fasta file containing the genome. |
| --gtf [file] | Path to the gtf file containing the genome annotation. |
| --skipBC [boolean] | Skip basecalling step. If truen give fastq folder as input. Default: true. |
| --flowcell [str] | Nanopore flowcell. Default = FLO-MIN106. |
| --kit [str] | Nanopore kit. Default = SQK-PBK004. |
| --gpu_mode [str] | Guppy basecaller configuration. Default: false. 
"gpu" mode is dedicated to NVIDIA Cuda compatible system according to Guppy specifications. |
| --min_qscore [float] | Minimum quality score threshold, default = 7.0. |
| --gpu_runners_per_device [int] | Number of runner per device, default = 32 (refer to guppy manual). |
| --num_callers [int] | Number of callers, default = 16 (refer to guppy manual). |
| --chunks_per_runner [int] | Number of chunks per runner, default = 512 (refer to guppy manual). |
| --chunks_size [int] | Chunck size, default = 1900 (refer to guppy manual). |
| --help --h | Display this help message. |

## Contributing

If you want to add more tools to this project, please read the [CONTRIBUTING.md](CONTRIBUTING.md).

## Authors

* **Xavier Grand** - *Maintainer*
* **Alia Rifki** - *Contributor*

## License

This project is licensed under the CeCiLL License- see the [LICENSE](LICENSE) file for details

## To Do:

* Give the user the possibility to choose the basecalling configuration file
* Change jpg to png in Start_positions.R
* Correct seqkit concatenate
